@@ -2,9 +2,11 @@
     resolution/conversion to XML.
 """
 import pickle
+from _pydevd_bundle.pydevd_constants import get_frame, get_current_thread_id, xrange
+
 
 from _pydev_imps._pydev_saved_modules import thread
-from _pydevd_bundle.pydevd_constants import get_frame, get_thread_id, xrange
+from _pydevd_bundle.pydevd_constants import get_frame, get_current_thread_id, xrange
 from _pydevd_bundle.pydevd_custom_frames import get_custom_frame
 from _pydevd_bundle.pydevd_xml import ExceptionOnEvaluate, get_type, var_to_xml
 
@@ -14,11 +16,16 @@ except ImportError:
     from io import StringIO
 import sys  # @Reimport
 
+try:
+    from collections import OrderedDict
+except:
+    OrderedDict = dict
+
 from _pydev_imps._pydev_saved_modules import threading
 import traceback
 from _pydevd_bundle import pydevd_save_locals
 from _pydev_bundle.pydev_imports import Exec, execfile
-from _pydevd_bundle.pydevd_utils import to_string
+from _pydevd_bundle.pydevd_utils import to_string, VariableWithOffset
 
 SENTINEL_VALUE = []
 
@@ -44,7 +51,7 @@ def _iter_frames(initialFrame):
 
 def dump_frames(thread_id):
     sys.stdout.write('dumping frames\n')
-    if thread_id != get_thread_id(threading.currentThread()):
+    if thread_id != get_current_thread_id(threading.currentThread()):
         raise VariableError("find_frame: must execute on same thread")
 
     curFrame = get_frame()
@@ -85,7 +92,7 @@ def get_additional_frames_by_id(thread_id):
 def find_frame(thread_id, frame_id):
     """ returns a frame on the thread that has a given frame_id """
     try:
-        curr_thread_id = get_thread_id(threading.currentThread())
+        curr_thread_id = get_current_thread_id(threading.currentThread())
         if thread_id != curr_thread_id:
             try:
                 return get_custom_frame(thread_id, frame_id)  # I.e.: thread_id could be a stackless frame id + thread_id.
@@ -137,13 +144,18 @@ def find_frame(thread_id, frame_id):
                 else:
                     msgFrames += '  -  '
 
-            errMsg = '''find_frame: frame not found.
-    Looking for thread_id:%s, frame_id:%s
-    Current     thread_id:%s, available frames:
-    %s\n
-    ''' % (thread_id, lookingFor, curr_thread_id, msgFrames)
-
-            sys.stderr.write(errMsg)
+# Note: commented this error message out (it may commonly happen 
+# if a message asking for a frame is issued while a thread is paused
+# but the thread starts running before the message is actually 
+# handled).
+# Leaving code to uncomment during tests.  
+#             err_msg = '''find_frame: frame not found.
+#     Looking for thread_id:%s, frame_id:%s
+#     Current     thread_id:%s, available frames:
+#     %s\n
+#     ''' % (thread_id, lookingFor, curr_thread_id, msgFrames)
+# 
+#             sys.stderr.write(err_msg)
             return None
 
         return frameFound
@@ -162,13 +174,13 @@ def getVariable(thread_id, frame_id, scope, attrs):
     BY_ID means we'll traverse the list of all objects alive to get the object.
 
     :attrs: after reaching the proper scope, we have to get the attributes until we find
-            the proper location (i.e.: obj\tattr1\tattr2)
+            the proper location (i.e.: obj\tattr1\tattr2).
 
     :note: when BY_ID is used, the frame_id is considered the id of the object to find and
            not the frame (as we don't care about the frame in this case).
     """
     if scope == 'BY_ID':
-        if thread_id != get_thread_id(threading.currentThread()):
+        if thread_id != get_current_thread_id(threading.currentThread()):
             raise VariableError("getVariable: must execute on same thread")
 
         try:
@@ -229,6 +241,23 @@ def getVariable(thread_id, frame_id, scope, attrs):
     return var
 
 
+def get_offset(attrs):
+    """
+    Extract offset from the given attributes.
+
+    :param attrs: The string of a compound variable fields split by tabs.
+      If an offset is given, it must go the first element.
+    :return: The value of offset if given or 0.
+    """
+    offset = 0
+    if attrs is not None:
+        try:
+            offset = int(attrs.split('\t')[0])
+        except ValueError:
+            pass
+    return offset
+
+
 def resolve_compound_variable_fields(thread_id, frame_id, scope, attrs):
     """
     Resolve compound variable in debugger scopes by its name and attributes
@@ -239,16 +268,24 @@ def resolve_compound_variable_fields(thread_id, frame_id, scope, attrs):
     :param attrs: after reaching the proper scope, we have to get the attributes until we find
             the proper location (i.e.: obj\tattr1\tattr2)
     :return: a dictionary of variables's fields
+
+    :note: PyCharm supports progressive loading of large collections and uses the `attrs`
+           parameter to pass the offset, e.g. 300\t\\obj\tattr1\tattr2 should return
+           the value of attr2 starting from the 300th element. This hack makes it possible
+           to add the support of progressive loading without extending of the protocol.
     """
+    offset = get_offset(attrs)
+
+    orig_attrs, attrs = attrs, attrs.split('\t', 1)[1] if offset else attrs
 
     var = getVariable(thread_id, frame_id, scope, attrs)
 
     try:
         _type, _typeName, resolver = get_type(var)
-        return _typeName, resolver.get_dictionary(var)
+        return _typeName, resolver.get_dictionary(VariableWithOffset(var, offset) if offset else var)
     except:
         sys.stderr.write('Error evaluating: thread_id: %s\nframe_id: %s\nscope: %s\nattrs: %s\n' % (
-            thread_id, frame_id, scope, attrs,))
+            thread_id, frame_id, scope, orig_attrs,))
         traceback.print_exc()
 
 
@@ -278,6 +315,10 @@ def resolve_compound_var_object_fields(var, attrs):
     :param attrs: a sequence of variable's attributes separated by \t (i.e.: obj\tattr1\tattr2)
     :return: a dictionary of variables's fields
     """
+    offset = get_offset(attrs)
+
+    attrs = attrs.split('\t', 1)[1] if offset else attrs
+
     attr_list = attrs.split('\t')
 
     for k in attr_list:
@@ -286,7 +327,7 @@ def resolve_compound_var_object_fields(var, attrs):
 
     try:
         type, _typeName, resolver = get_type(var)
-        return resolver.get_dictionary(var)
+        return resolver.get_dictionary(VariableWithOffset(var, offset) if offset else var)
     except:
         traceback.print_exc()
 
@@ -406,29 +447,29 @@ def change_attr_expression(thread_id, frame_id, attr, expression, dbg, value=SEN
             if result:
                 return result
 
+        if value is SENTINEL_VALUE:
+            # It is possible to have variables with names like '.0', ',,,foo', etc in scope by setting them with
+            # `sys._getframe().f_locals`. In particular, the '.0' variable name is used to denote the list iterator when we stop in
+            # list comprehension expressions. This variable evaluates to 0. by `eval`, which is not what we want and this is the main
+            # reason we have to check if the expression exists in the global and local scopes before trying to evaluate it.
+            value = frame.f_locals.get(expression) or frame.f_globals.get(expression) or eval(expression, frame.f_globals, frame.f_locals)
+
         if attr[:7] == "Globals":
             attr = attr[8:]
             if attr in frame.f_globals:
-                if value is SENTINEL_VALUE:
-                    value = eval(expression, frame.f_globals, frame.f_locals)
                 frame.f_globals[attr] = value
                 return frame.f_globals[attr]
         else:
             if pydevd_save_locals.is_save_locals_available():
-                if value is SENTINEL_VALUE:
-                    value = eval(expression, frame.f_globals, frame.f_locals)
                 frame.f_locals[attr] = value
                 pydevd_save_locals.save_locals(frame)
                 return frame.f_locals[attr]
 
             # default way (only works for changing it in the topmost frame)
-            if value is SENTINEL_VALUE:
-                value = eval(expression, frame.f_globals, frame.f_locals)
             result = value
             Exec('%s=%s' % (attr, expression), frame.f_globals, frame.f_locals)
             return result
 
-
     except Exception:
         traceback.print_exc()
 
@@ -479,6 +520,10 @@ def array_to_xml(array, name, roffset, coffset, rows, cols, format):
     return xml
 
 
+class ExceedingArrayDimensionsException(Exception):
+    pass
+
+
 def array_to_meta_xml(array, name, format):
     type = array.dtype.kind
     slice = name
@@ -502,7 +547,7 @@ def array_to_meta_xml(array, name, format):
     l = len(array.shape)
     reslice = ""
     if l > 2:
-        raise Exception("%s has more than 2 dimensions." % slice)
+        raise ExceedingArrayDimensionsException()
     elif l == 1:
         # special case with 1D arrays arr[i, :] - row, but arr[:, i] - column with equal shape and ndim
         # http://stackoverflow.com/questions/16837946/numpy-a-2-rows-1-column-file-loadtxt-returns-1row-2-columns
@@ -600,8 +645,10 @@ def dataframe_to_xml(df, name, roffset, coffset, rows, cols, format):
     def col_to_format(c):
         return format if dtypes[c] == 'f' and format else array_default_format(dtypes[c])
 
+    iat = df.iat if dim == 1 or len(df.columns.unique()) == len(df.columns) else df.iloc
+
     xml += header_data_to_xml(rows, cols, dtypes, col_bounds, col_to_format, df, dim)
-    xml += array_data_to_xml(rows, cols, lambda r: (("%" + col_to_format(c)) % (df.iat[r, c] if dim > 1 else df.iat[r])
+    xml += array_data_to_xml(rows, cols, lambda r: (("%" + col_to_format(c)) % (iat[r, c] if dim > 1 else iat[r])
                                                     for c in range(cols)))
     return xml
 
